@@ -21,20 +21,28 @@
 #'   dependencies, you can tell R to use it by pointing reticulate at it,
 #'   commonly by setting an environment variable:
 #'
-#'   ``` Sys.setenv("RETICULATE_PYTHON" = "~/path/to/python-env/bin/python") ```
+#'   ``` R
+#'   Sys.setenv("RETICULATE_PYTHON" = "~/path/to/python-env/bin/python")
+#'   ```
 #'
 #' @section Apple Silicon: Tensorflow on Apple Silicon is not officially
-#'   supported by the tensorflow maintainers. It is known that there can be
-#'   issues running the official Tensorflow package under Rosetta as well.
-#'   Fortunately, for the time being Apple has published a custom version of
-#'   Tensorflow compatible with M1 macs. Installation instructions can be found
-#'   at: \url{https://developer.apple.com/metal/tensorflow-plugin/}. Please note
-#'   that this is an experimental build of both python and tensorflow. After
-#'   following the instructions provided by Apple, you can advise reticulate to
-#'   use that python installation by placing the following in your `.Renviron`
-#'   file:
+#'   supported by the Tensorflow maintainers. However Apple has published a
+#'   custom version of Tensorflow compatible with Arm Macs.
+#'   `install_tensorflow()` will install the special packages `tensorflow-macos`
+#'   and `tensorflow-metal` on Arm Macs. See
+#'   \url{https://developer.apple.com/metal/tensorflow-plugin/} for instructions
+#'   on how to do the equivalent manually. Please note that this is an
+#'   experimental build of both Python and Tensorflow, with known issues. In
+#'   particular, certain operations will cause errors, but can often be remedied
+#'   by pinning them to the CPU. For example:
 #'
-#'   ``` RETICULATE_PYTHON = "~/miniforge3/bin/python" ```
+#'   ```` R
+#'   x <- array(runif(64*64), c(1, 64, 64))
+#'   keras::layer_random_rotation(x, .5)  # Error:
+#'   # No registered 'RngReadAndSkip' OpKernel for 'GPU' devices
+#'   # Pin the operation to the CPU to avoid the error
+#'   with(tf$device("CPU"), keras::layer_random_rotation(x, .5) ) # No Error
+#'   ````
 #'
 #' @section Additional Packages:
 #'
@@ -74,9 +82,11 @@
 #' @param restart_session Restart R session after installing (note this will
 #'   only occur within RStudio).
 #'
-#' @param python_version,conda_python_version the python version installed in
-#'   the created conda environment. Ignored when attempting to install with a
-#'   Python virtual environment.
+#' @param python_version,conda_python_version Pass a string like "3.8" to
+#'   request that conda install a specific Python version. This is ignored when
+#'   attempting to install in a Python virtual environment. Note that the Python
+#'   version must be compatible with the requested Tensorflow version, documented
+#'   here: <https://www.tensorflow.org/install/pip#system-requirements>
 #'
 #' @param pip_ignore_installed Whether pip should ignore installed python
 #'   packages and reinstall all already installed python packages. This defaults
@@ -95,18 +105,31 @@ install_tensorflow <- function(method = c("auto", "virtualenv", "conda"),
                                envname = NULL,
                                extra_packages = NULL,
                                restart_session = TRUE,
-                               conda_python_version = "3.7",
+                               conda_python_version = NULL,
                                ...,
                                pip_ignore_installed = TRUE,
                                python_version = conda_python_version) {
 
+  method <- match.arg(method)
+
+  if(is_mac_arm64())
+    return(install_tensorflow_mac_arm64(
+      method = method,
+      conda = conda,
+      version = version,
+      envname = envname,
+      extra_packages = extra_packages,
+      restart_session = restart_session,
+      python_version = python_version
+    ))
+
+
   # verify 64-bit
   if (.Machine$sizeof.pointer != 8) {
     stop("Unable to install TensorFlow on this platform.",
          "Binary installation is only available for 64-bit platforms.")
   }
 
-  method <- match.arg(method)
 
   # some special handling for windows
   if (is_windows()) {
@@ -165,7 +188,7 @@ install_tensorflow <- function(method = c("auto", "virtualenv", "conda"),
 }
 
 
-default_version <- numeric_version("2.7")
+default_version <- numeric_version("2.8")
 
 parse_tensorflow_version <- function(version) {
   # returns unquoted string directly passable to pip, e.g 'tensorflow==2.5.*'
@@ -175,6 +198,9 @@ parse_tensorflow_version <- function(version) {
 
   version <- as.character(version) # if numeric_version()
 
+  if(version == "release-cpu")
+    return("tensorflow-cpu")
+
   # full path to whl.
   if (grepl("^.*\\.whl$", version))
     return(normalizePath(version))
@@ -220,3 +246,78 @@ install_tensorflow_extras <- function(packages, conda = "auto") {
           "Use the extra_packages argument to install_tensorflow() or reticulate::py_install() to ",
           "install additional packages.")
 }
+
+install_tensorflow_mac_arm64 <- function(method = c("auto", "virtualenv", "conda"),
+                                         conda = "auto",
+                                         version = "default",
+                                         envname = NULL,
+                                         extra_packages = NULL,
+                                         restart_session = TRUE,
+                                         python_version = NULL) {
+  stopifnot(is_mac_arm64())
+  method <- match.arg(method)
+
+  if(method == "virtualenv")
+    stop("Tensorflow on Arm Macs only supported in Conda Python.",
+         " Install Miniconda with `reticulate::install_miniconda()`")
+
+  have_conda <- !is.null(tryCatch(conda_binary(conda), error = function(e) NULL))
+  if (!have_conda) {
+    stop("Tensorflow installation failed (no conda binary found)\n\n",
+         "Install Miniconda by running `reticulate::install_miniconda()`",
+         "before installing Tensorflow.\n",
+         call. = FALSE)
+  }
+
+  if(!version %in% c("default", "release"))
+    warning(
+      "Only `version = 'default'` supported on Arm Macs at this time.",
+      "Request for Tensorflow version '", version, "' ignored.")
+
+  conda <- conda_binary(conda)
+
+  install <- function(pkg, ...) {
+    message("Installing: ", paste(pkg, collapse = ", "))
+    py_install(pkg, ...,  method = "conda", conda = conda,
+               envname = envname, python_version = python_version)
+  }
+
+  extra_packages <- setdiff(extra_packages, c("scipy", "requests", "h5py"))
+  # "scipy","requests", "h5py" # pinned versions pulled in by "tensorflow-deps" already
+  # "pyyaml" no longer needed
+
+  # split extra_packages into pre and post install lists
+  # dangerous to invoke conda after pip, so everything conda goes first,
+  # with official tensorflow-* packages going last so their pinned dependancy version wins.
+  # packages like h5py o Pillow not available via pip yet for arm mac
+  # and tensorflow-hub via conda pulls in the wrong dependencies, so must come from pip.
+  extra_packages_pip <- intersect(extra_packages, c("tensorflow-hub",
+                                                    "tensorflow-datasets"))
+  extra_packages_conda <- setdiff(extra_packages, extra_packages_pip)
+
+  if(length(extra_packages_conda))
+    install(extra_packages_conda) # keras passes: c("pandas", "Pillow")
+
+  os_ver <- numeric_version(gsub("macOS [a-zA-Z ]*([0-9.]+)", "\\1",
+                                 utils::sessionInfo()$running, perl = TRUE))
+
+  if (os_ver < "12") {
+    install("tensorflow-deps==2.6", channel = c("apple", "conda-forge"))
+    install("tensorflow-macos==2.6.0", pip = TRUE)
+    install("tensorflow-metal==0.2.0", pip = TRUE)
+  } else {
+    install("tensorflow-deps", channel = c("apple", "conda-forge"))
+    install("tensorflow-macos", pip = TRUE)
+    install("tensorflow-metal", pip = TRUE)
+  }
+
+  if(length(extra_packages_pip))
+    install(extra_packages_pip, pip = TRUE)
+
+  if (restart_session &&
+      requireNamespace("rstudioapi", quietly = TRUE) &&
+      rstudioapi::hasFun("restartSession"))
+    rstudioapi::restartSession()
+
+  invisible()
+}
